/////////////////////////////////////////////////////////////////
/////	Mapping from ASCII to C64 character set of 512  /////
/////		ROM characters.  ////////////////////
/////////////////////////////////////////////////////////////////
/////Implementation File				   //////
/////////////////////////////////////////////////////////////////

#include "CharMap.h"
#include "char.h"
#include <vga.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

void DrawStr (char * Str, int X, int Y, char Col)
{
	int DStr_buf[80];
	int Len;
	int i;
	Len = strlen(Str);
	if (Len > 80) Len = 80;
	for (i=0;i<Len;i++)
	{
		DStr_buf[i] = ASC2C64(Str[i])|(Col<<16);
	}
	DrawCharAttrRow(DStr_buf, X, Y, Len);
}


void DrawAscChar (int C, int X, int Y, char Col)
{
	DrawChar (ASC2C64(C), X, Y, Col);
}
// sends bigger segments than DrawChar, might be faster //
// limited to only one screen line - 8 horiz. segments  //
static char * DCAR_Segs[8];

void InitDCAR_Segs()
{
	int i;
	for (i=0;i<8;i++) DCAR_Segs[i] = malloc(640);
}


void DrawCharAttrRow (int * CA, int X, int Y, int L)
{
	static int first = 1;
	unsigned int fore, back;
	int N, i, j, C, b;
	unsigned int Len;
	if (first) {InitDCAR_Segs(); first = 0;}
	Len = (L > (80 - X))?80-X:L; //clip to screen //
	for (N=0;N<Len;N++)
	{
		fore = (CA[N] & 0xF00000) >> 20;
		back = (CA[N] & 0x0F0000) >> 16;
		C = CA[N] & 0x1FF;
		for (i=0;i<8;i++)
		{
			b = C64Set[i + C * 8];
			for (j=0;j<8;j++)
			{
			if (b & 0x80) *(DCAR_Segs[i]+ N*8 + j) = fore;
			else	*(DCAR_Segs[i] + N*8 + j) = back;
			b <<= 1;
			}
		}
	}
	for (i=0;i<8;i++)
		vga_drawscansegment(DCAR_Segs[i], 
			X*8, Y*8 + i,Len*8);


}

void DrawChar (int C, int X, int Y, char Col)
{
	int CA;
	CA = (C&0x1FF)|(Col<<16);
	DrawCharAttrRow(&CA, X, Y, 1);
}



typedef struct _CMAP
{
	char AscStart;
	int  C64Start;
	int	 Len;
} CMAP;

#define NUMCMAP 10

CMAP  CMap [NUMCMAP] =
{
	{  '@',	0x0,	27}, 
	{  'a',	0x101,	26},
	{  ' ', 0x20,	32},
	{  '_', 0x79,	1 },
	{  '[', 0x1B,	1 },
	{  ']', 0x1D,	2 },
	{  '\\', 0x4D,	1 },
	{  '{', 0x6B,	1 },
	{  '}', 0x73,	1 },
	{  '|', 0x24,	1 }
};

int C642ASC (int  T)
{
	int i, j;
	for (i=0;i<NUMCMAP;i++)
	{
		j = T - CMap[i].C64Start;
		if ( (j < CMap[i].Len) && (j >= 0) )
			return (CMap[i].AscStart + j);
	}
	return 0x20; //space
}

	
int ASC2C64 (char C)
{
	int i;
	int j;
	for (i=0;i<NUMCMAP;i++)
	{
		j = C - CMap[i].AscStart;
		if ( (j < CMap[i].Len) && (j >= 0) )
			return (CMap[i].C64Start + j);
	}
	return 0x66; //checkerboard
}
