#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <linux/soundcard.h>
#include <unistd.h>


#include "cardfeed.h"


int fs[6]={128, 256, 512, 1024, 2048, 4096};

int buf_size;
char *buf;
static int _16b=1;

static int fd;
static int sound_opened=0;


/* return 0 if ok, -1 on error and print what's wrong */
int open_sound(int freq, int bits, int stereo, int nb_frag, int frag_buf_size)
{
  /* inspired from nagraudio (http://asterix.uc3m.es/nagraudio) */
  int aux, n;
  int caps;
  
  if (sound_opened) {
    fprintf(stderr, "Oops. Sound already opened, pal ! What the heck are ya doin' ?\n");
    return -1;
  }
  
  if ((fd=open("/dev/dsp", O_RDWR))==-1) {
    perror("open audio");
    return -1;
  }

  for (n=0; n<6; n++)
    if (fs[n]==frag_buf_size)
      break;
  aux=(nb_frag<<16)|(n+7);
  if (ioctl(fd, SNDCTL_DSP_SETFRAGMENT, &aux)<0) {
    perror("audio: set fragments");
    close(fd);
    return -1;
  }

  if (ioctl(fd, SNDCTL_DSP_SETDUPLEX, 0)==-1) {
    perror("setting duplex on audio");
    close(fd);
    return -1;
  }

  if (ioctl(fd, SNDCTL_DSP_GETCAPS, &caps)==-1) {
    perror("getting capabilities of audio card");
    close(fd);
    return -1;
  }
  if (!(caps & DSP_CAP_DUPLEX)) {
    fprintf(stderr, "Sorry but your soundcard can't do full duplex\n");
    close(fd);
    return -1;
  }

  if (bits==8) {
    aux=AFMT_U8;
    _16b=0;
  } else
    aux=AFMT_S16_LE;
  if (ioctl(fd,SNDCTL_DSP_SETFMT, &aux) == -1) {
    perror("audio: set format");
    close(fd);
    return -1;
  }

  aux=stereo;
  if (ioctl(fd,SNDCTL_DSP_STEREO, &aux) == -1) {
    perror("audio: set format");
    close(fd);
    return -1;
  }

  aux=freq;
  if (ioctl(fd,SNDCTL_DSP_SPEED, &aux) == -1) {
    perror("audio: set format");
    close(fd);
    return -1;
  }

  buf_size=frag_buf_size;
  if (!(buf=(char *)malloc(buf_size))) {
    perror("allocating buffer's memory");
    close(fd);
    return -1;
  }

  sound_opened=1;

  return 0;
}

void close_sound(void)
{
  close(fd);
  free(buf);
  sound_opened=0;
}

/* return -1 on error 0 otherwise. Write waht was wrong on stderr. */
int read_sound(void)
{
  if (read(fd, buf, buf_size)!=buf_size) {
    perror("reading from audio card");
    return -1;
  }
  return 0;
}

/* return -1 on error 0 otherwise. Write what was wrong on stderr. */
int write_sound(void)
{
  if (write(fd, buf, buf_size)!=buf_size) {
    perror("writing to audio card");
    return -1;
  }
  return 0;
}
