/////////////////////////////////////////////////////////////////////////////////////////////////////////////
////	Storage for effects /////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 8 effect boxes in a row
#define FX_ROW 8

typedef struct _effect_type
{
	void * func;
	char short_desc[3];
	char long_desc[16];
	int nIn;
	int nOut;
	int nParms;
	char parm_desc[4][16];
	int parm_range[4];

	
} effect_type;

typedef void (*PumpFunc) (int, int);


typedef struct _effect_stat
{
	int type;
	int parms[4];
	int parmlink[4];

	int link_is[2];
	int link_Col[2]; //link_Col==FX_ROW means out loop terminal!
	int link_Loop[2];
	int link_Term[2];

	short * out_buf[2];
	short * in_buf[2];
	
	void * extra;
} effect_stat;


typedef struct _loopbus_stat
{
	int is_linked;
	int Col;
	int Loop;
	int Term;
} loopbus_stat;

void fx_init();

void fx_initmutex();
void fx_lock();
void fx_unlock();
int fx_islocked();

void fx_remove(int Col, int Loop);
int fx_gettype(int Col, int Loop);
void fx_new(int Col, int Loop, int Type);

void fx_disconnect_out(int Col, int Loop, int Term);
void fx_disconnect_in(int Col, int Loop, int Term);
void fx_disconnect_all(int Col, int Loop);

typedef struct _pixel
{
	int X;
	int Y;
} pixel;





#ifdef INIT_GLOBALS

effect_stat fx_layout[NUM_LOOPS][FX_ROW];

loopbus_stat loopbus_links[NUM_LOOPS];



const int nEffectTypes = 9;

const effect_type effect_types[] = 
{
	{
	Eff_Distortion,
	"DS", "DISTORTION",
	1, 1, 3,	{"Drive", "Tone", "Level", ""},
			{1000, 1000, 1000, 0}
	},
	{
	Eff_Overdrive,
	"OD", "OVERDRIVE",
	1, 1, 2,	{"Drive", "Level", "", ""},
			{1000, 1000, 1000, 1000}
	},
	{
	Eff_Equalization,
	"PQ", "PARAM. EQ.",
	1, 1, 3,	{"Mode", "Freq", "Resonance", ""},
			{2, 1000, 1000, 0}
	},
	{
	Eff_Echo,
	"EC", "ECHO",
	1, 1, 4,	{"Wet", "Dry", "Delay", "Regen"},
			{1000, 1000, 1000, 1000}
	},
	{
	Eff_Chorus,
	"CH", "CHORUS/FLANGE",
	1, 1, 4,	{"Wet", "Speed", "Depth", "Regen"},
			{1000, 1000, 1000, 1000}
	},
	{
	Eff_Octave,
	"OV", "OCTAVE",
	1, 1, 1,	{"Wet", "", "", ""},
			{1000, 1000, 1000, 1000}
	},
	{
	Eff_Split,
	"SP", "SPLITTER",
	1, 2, 0, 	{"", "", "", ""},
			{0, 0, 0, 0}
	},
	{
	Eff_Mix,
	"MX", "MIXER",
	2, 1, 2,	{"Volume 1", "Volume 2", "", ""},
			{1000, 1000, 1000, 1000}
	},
	{
	Eff_Modulator,
	"MD", "MODULATOR",
	2, 1, 2,	{"Bias", "Level"},
			{1000, 1000, 0, 0}
	}
	
	
};


	
#else

extern const effect_type effect_types[];
extern effect_stat fx_layout[NUM_LOOPS][FX_ROW];
extern const int nEffectTypes;
extern loopbus_stat loopbus_links[];
#endif
