#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>
#include <pthread.h>
#include <semaphore.h>
#include <errno.h>
#include <vga.h>
#include "CharMap.h"
#include "loopdata.h"
#include "input.h"
#include "gui.h"
#include "fxdata.h"

static pthread_mutex_t fx_mutex;

void fx_initmutex()
{
	pthread_mutex_init(&fx_mutex, NULL);
}


void fx_lock()
{
        pthread_mutex_lock(&fx_mutex);
}

void fx_unlock()
{
	pthread_mutex_unlock(&fx_mutex);
}

int fx_islocked()
{
	int ret;
	ret = pthread_mutex_trylock(&fx_mutex);
	if (ret == 0) pthread_mutex_unlock(&fx_mutex);
	return (ret != 0);
}



void fx_remove(int Col, int Loop)
{
	fx_lock();
	fx_layout[Loop][Col].type = -1;
	fx_unlock();
}

int fx_gettype(int Col, int Loop)
{
	return fx_layout[Loop][Col].type;
}

void fx_init()
{
	int i, j;
	for (i=0;i<NUM_LOOPS;i++)
		for (j=0;j<FX_ROW;j++)
			fx_layout[i][j].type = -1;
	for (i=0;i<NUM_LOOPS;i++)
	{
		loopbus_links[i].is_linked = 0;
		for (j=0;j<FX_ROW;j++)
		{	
			fx_layout[i][j].link_is[0]=
				fx_layout[i][j].link_is[1] = 0;
			fx_layout[i][j].extra = NULL;
			// don't initialize effect buffers here!
			// that has to be done in the SCHED_FIFO
			// thread, after the sound is opened,
			// so that we'll all know how big the 
			// audio buffer is.
			// this initializer only does stuff that the
			// GUI thread uses.
		}
		
			
	}
		
	fx_initmutex();
}

void fx_new(int Col, int Loop, int Type)
{
	int i;
	fx_remove(Col, Loop);
	fx_lock();
	fx_layout[Loop][Col].type = Type;
	for (i=0;i<4;i++) 
	{
		fx_layout[Loop][Col].parms[i]=0;
		fx_layout[Loop][Col].parmlink[i]=-1;
		if (fx_layout[Loop][Col].extra != NULL)
			{free(fx_layout[Loop][Col].extra);}
		fx_layout[Loop][Col].extra = NULL;
	}
	
	fx_unlock();
}


void fx_disconnect_out(int Col, int Loop, int Term)
{
	// this one is very easy //
	fx_layout[Loop][Col].link_is[Term]=0;
}

void fx_disconnect_in(int Col, int Loop, int Term)
{
	// this one is not as easy //
	int L, C, T;
	for (L=0;L<NUM_LOOPS;L++)
		for (C=0;C<FX_ROW;C++)
			for (T=0;T<2;T++)
	{
		if (fx_layout[L][C].link_is[T]==0) continue;
		if (fx_layout[L][C].link_Loop[T]!=Loop) continue;
		if (fx_layout[L][C].link_Col[T]!=Col) continue;
		if (Col != FX_ROW) // link_Term doesn't matter for output terminal
			if (fx_layout[L][C].link_Term[T]!=Term) continue;
		fx_layout[L][C].link_is[T]=0; return;
	}
	for (L=0;L<NUM_LOOPS;L++)
	{
		if (loopbus_links[L].is_linked==0) continue;
		if (loopbus_links[L].Col != Col) continue;
		if (loopbus_links[L].Loop != Loop) continue;
		if (loopbus_links[L].Term != Term) continue;
		loopbus_links[L].is_linked = 0;
	}
	
}

void fx_disconnect_all(int Col, int Loop)
{
	int T;
	for (T = 0; T < 2; T++)
	{
		fx_disconnect_out(Col,Loop,T);
		fx_disconnect_in(Col,Loop,T);
	}
	
}
	
